#!/usr/bin/env ruby

# Copyright (c) 2010 Microsoft Corp.
# License: ?

require 'rubygems'
require 'ruby-aws'
require 'digest/md5'
require 'include/config'
require 'include/proxy'
require 'include/requester'
require 'include/templates'

@config = MOS::Config.create
@mturk  = Amazon::WebServices::MechanicalTurkRequester.new :Host => @config[:Server]

def getHITUrl(hitTypeId)
	if @mturk.host =~ /sandbox/
		"http://workersandbox.mturk.com/mturk/preview?groupId=#{hitTypeId}" # Sandbox Url
	else
		"http://mturk.com/mturk/preview?groupId=#{hitTypeId}" # Production Url
	end
end

def createHITs
	input = Amazon::Util::DataReader.load(@config[:HITInputFile], :Tabular)
	props = Amazon::Util::DataReader.load(@config[:HITPropertiesFile], :Properties)

	input.each do |line|
		line[:identifier] = CGI.escape(line[:identifier].to_s)
	end

	quest = MOS.buildQuestionTemplateExternal(@options[:external_question])

	total_cost = props[:Reward][:Amount] * input.size * props[:MaxAssignments]
	total_cost *= 1.10	# Amazon's commission is 10%
	available_funds = @mturk.availableFunds

	if available_funds < total_cost
		$stderr.printf("Not enough funds: operation requires %.2f USD, account contains %.2f USD.\n",
				total_cost, available_funds)
		exit
	end

	printf("Create %d HITs with %d assignments each for a total cost of %.2f %s? (y/n) ",
			input.size, props[:MaxAssignments], total_cost, props[:Reward][:CurrencyCode])
	response = STDIN.gets.chomp.downcase
	if response != 'y'
		printf("Exiting...\n")
		exit
	end

	puts("Creating HITs...")
	hits = @mturk.createHITs(props, quest, input)
	puts("Done.")

	hit_ids = hits[:Created].collect { |h| h[:HITId] }
	hit_type_id = hits[:Created].first[:HITTypeId]

	puts("Created HITs: #{hit_ids.join(' ')}")
	puts("Url: #{getHITUrl(hit_type_id)}")

	Amazon::Util::DataReader.save(@config[:HITSuccessFile], hits[:Created], :Tabular)
	Amazon::Util::DataReader.save(@config[:HITFailureFile], hits[:Failed],  :Tabular)
end

@options = { }
optparse = OptionParser.new do |opts|
	opts.banner = "Usage: #{$0} [options]"

	@options[:external_question] = nil
	opts.on('--external-question URL',
		'Generate HITs for an externally hosted question, accessible at URL (required)' ) do |url|
		@options[:external_question] = url
	end

	opts.on( '-h', '--help', 'Display this screen' ) do
		puts opts
		exit
	end

	opts.on_tail("")
	opts.on_tail("This script submits HITs by instantiating the HIT template with input parameters.")
end

begin
	optparse.parse!
rescue OptionParser::ParseError => e
	$stderr.puts("Error parsing command-line arguments: " + e.message)
	$stderr.puts("Try '#{$0} --help' for more information.")
	exit
end

if !@options[:external_question]
	$stderr.puts("Error: external HIT URL not supplied.")
	$stderr.puts("Try '#{$0} --help' for more information.")
	exit
end

createHITs

