#!/usr/bin/env ruby

require 'include/Solution'

def percentile(numbers, pct)
	numbers = numbers.sort
	h = pct * (numbers.length - 1)
	numbers[h.round]
end

if ARGV.length < 2
	puts "Usage: #{$0} videoId <source xml>"
else
	videoId = ARGV[0]
	src_xml = ARGV[1]
	dst_xml = src_xml

	solution_list = Solution::get_all(src_xml, videoId)

	# ITER 1
	solution_list.each { |s| s.positiveCoordinates }
	maxW = solution_list.length
	maxT = solution_list.map { |s| s.timestamp.length }.max 

	d = Array.new(maxW).fill(0)
	maxT.times do |t|
		maxW.times do |i|
			distance = Array.new

			maxW.times do |j|
				next if i == j

				r = solution_list[i]
				s = solution_list[j]

				distance << Math.hypot(r.coordX[t] - s.coordX[t], r.coordY[t] - s.coordY[t])
			end

			d[i] += distance.min
		end
	end

	maxW.times do |i|
		d[i] /= maxT.to_f
		printf("%s: %2.2f\n", solution_list[i].assignmentId, d[i])
	end

	d80 = percentile(d, 0.80)	# eliminate answers in the upper 80% percentile
	printf("Disp. 80%% percentile: %2.2f\n\n", d80)

	maxW.times do |i|
		if d[i] > d80
			printf("Deleting %s: %2.2f\n", solution_list[i].assignmentId, d[i])
			solution_list[i].valid = false
		end
	end

	solution_list.delete_if { |s| !s.valid }
	print "\n"

	# ITER 2
	if true
		solution_list.each { |s| s.discardStationary }
		maxW = solution_list.length
		maxT = solution_list.map { |s| s.timestamp.length }.max 

		maxT.times do |t|
			maxW.times do |i|
				count_total = 0
				count_valid = 0

				maxW.times do |j|
					next  if i == j
					break if count_valid > 0

					r = solution_list[i]
					s = solution_list[j]

					for u in (t-5)..(t+5) do
						if u >= 0 && u < maxT &&
						   r.coordX[t] >= 0 && r.coordY[t] >= 0 &&
						   s.coordX[u] >= 0 && s.coordY[u] >= 0
							deltaX = (r.coordX[t] - s.coordX[u]).abs
							deltaY = (r.coordY[t] - s.coordY[u]).abs

							count_total += 1
							if deltaX < 40 && deltaY < 30
								count_valid += 1
								break
							end
						end
					end


				end

				if count_total > 0 && count_valid == 0
					r = solution_list[i]

					for u in (t-10)..(t+10) do
						if u >= 0 && u < maxT
							r.coordX[u] = -1
							r.coordY[u] = -1
						end
					end
				end
			end
		end
	end

	Solution::remove_all(dst_xml, videoId)
	Solution::add(dst_xml, videoId, solution_list)
end

