#!/usr/bin/env ruby

# Copyright (c) 2010 Microsoft Corp.
# License: ?

require 'rubygems'
require 'ruby-aws'
require 'facets/array'

module MOS

def parseAnswer(answer_xml)
	answer = { }

	xml_doc = REXML::Document.new(answer_xml)
	xml_doc.root.each_element('//Answer') do |a|
		key = a.elements['QuestionIdentifier'].text.to_sym

		if a.elements['SelectionIdentifier'] != nil
			value = a.elements['SelectionIdentifier'].text.to_s
		elsif a.elements['OtherSelectionText'] != nil
			value = a.elements['OtherSelectionText'].text.to_s
		elsif a.elements['FreeText'] != nil
			value = a.elements['FreeText'].text.to_s
		else
			value = ''
		end

		answer[key] = value.gsub(/[\t"]/, ' ')	# replace tabs and quotes, because answer with be saved in tabular format
	end

	answer
end

def indexByInteger(input, key)
	output = []
	input.each_pair do |val, h|
		h = h.clone
		h[key] = val
		output << h
	end
	output
end

def indexByUniqueKey(input, key)
	output = { }
	input.each do |item|
		item = item.clone
		val  = item.delete(key).to_sym
		output[val] = item
	end
	output
end

def indexByNonUniqueKey(input, key)
	output = { }
	input.each do |item|
		item = item.clone
		val  = item.delete(key).to_sym
		(output[val] ||= []) << item
	end
	output
end

def buildQuestionTemplateExternal(question_url)
	template = <<END
<?xml version="1.0"?>
<ExternalQuestion xmlns="http://mechanicalturk.amazonaws.com/AWSMechanicalTurkDataSchemas/2006-07-14/ExternalQuestion.xsd">
	<ExternalURL>#{question_url}?videoId=<%= @identifier %></ExternalURL>
	<FrameHeight>800</FrameHeight>
</ExternalQuestion>
END

	template
end

module_function :parseAnswer
module_function :indexByInteger
module_function :indexByUniqueKey
module_function :indexByNonUniqueKey
module_function :buildQuestionTemplateExternal

end # MOS
